from enum import Enum
from plum import dispatch
from typing import TypeVar, Union, Generic, List, Tuple
from spire.pdf.common import *
from spire.pdf import *
from ctypes import *
import abc

class Collections_PdfCollection(SpireObject):
    """
    A collection specifies the viewing and organizational characteristics
    of portable collections. The intent of portable collections is to present,
    sort, and search collections of related documents, such as email archives,
    photo collections, and engineering bidsets.
    """

    @property
    def Folders(self) -> 'PdfFolder':
        """
        (Required if the collection has folders; ExtensionLevel3)
        An indirect reference to the folder dictionary that is the
        single common ancestor of all other folders in a portable
        collection.
        """
        GetDllLibPdf().Collections_PdfCollection_get_Folders.argtypes = [c_void_p]
        GetDllLibPdf().Collections_PdfCollection_get_Folders.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().Collections_PdfCollection_get_Folders,self.Ptr)
        ret = None if intPtr == None else PdfFolder(intPtr)
        return ret

    @property
    def AssociatedFiles(self) -> List['PdfAttachment']:
        """
        Get the document collection associated files
        """
        GetDllLibPdf().Collections_PdfCollection_get_AssociatedFiles.argtypes = [c_void_p]
        GetDllLibPdf().Collections_PdfCollection_get_AssociatedFiles.restype = IntPtrArray
        intPtrArr = CallCFunction(GetDllLibPdf().Collections_PdfCollection_get_AssociatedFiles,self.Ptr)
        ret = None if intPtrArr == None else GetObjVectorFromArray(intPtrArr, PdfAttachment)
        return ret

    @dispatch
    def AddFile(self, filePath: str):
        """
        Add a local file.
        :param filePath: The local file path.
        """
        GetDllLibPdf().Collections_PdfCollection_AddFile.argtypes = [c_void_p, c_wchar_p]
        CallCFunction(GetDllLibPdf().Collections_PdfCollection_AddFile,self.Ptr, filePath)

    @dispatch
    def AddFile(self, fileName: str, stream: Stream):
        """
        Add a stream.
        :param fileName: The file name of the stream.
        :param stream: The stream.
        """
        intPtrstream: c_void_p = stream.Ptr
        GetDllLibPdf().Collections_PdfCollection_AddFileFS.argtypes = [c_void_p, c_wchar_p, c_void_p]
        CallCFunction(GetDllLibPdf().Collections_PdfCollection_AddFileFS,self.Ptr, fileName, intPtrstream)

    def AddAttachment(self, attachment: 'PdfAttachment'):
        """
        Add an attachment.
        :param attachment: The attachment.
        """
        intPtrattachment: c_void_p = attachment.Ptr
        GetDllLibPdf().Collections_PdfCollection_AddAttachment.argtypes = [c_void_p, c_void_p]
        CallCFunction(GetDllLibPdf().Collections_PdfCollection_AddAttachment,self.Ptr, intPtrattachment)

    def AddCustomField(self, fieldName: str, displayText: str, fieldType: 'CustomFieldType'):
        """
        Add a custom field.
        :param fieldName: Custom field name.
        :param displayText: Custom field display name.
        :param fieldType: Custom field type.
        """
        enumfieldType: c_int = fieldType.value
        GetDllLibPdf().Collections_PdfCollection_AddCustomField.argtypes = [c_void_p, c_wchar_p, c_wchar_p, c_int]
        CallCFunction(GetDllLibPdf().Collections_PdfCollection_AddCustomField,self.Ptr, fieldName, displayText, enumfieldType)

    def AddFileRelatedField(self, fieldName: str, displayText: str, fieldType: 'FileRelatedFieldType'):
        """
        Add a file related field.
        :param fieldName: File related field name.
        :param displayText: File related field display name.
        :param fieldType: File related field type.
        """
        enumfieldType: c_int = fieldType.value
        GetDllLibPdf().Collections_PdfCollection_AddFileRelatedField.argtypes = [c_void_p, c_wchar_p, c_wchar_p, c_int]
        CallCFunction(GetDllLibPdf().Collections_PdfCollection_AddFileRelatedField,self.Ptr, fieldName, displayText, enumfieldType)

    def Sort(self, fieldNames: List[str], order: List[bool]):
        """
        Sort embedded files with field names.
        :param fieldNames: The names of fields that the PDF viewer application
        uses to sort the items in the collection.
        :param order: Specifies whether the items in the collection are sorted
        in ascending order.
        """
        countfieldNames = len(fieldNames)
        ArrayTypefieldNames = c_wchar_p * countfieldNames
        arrayfieldNames = ArrayTypefieldNames()
        for i in range(0, countfieldNames):
            arrayfieldNames[i] = fieldNames[i]

        countorder = len(order)
        ArrayTypeorder = c_int * countorder
        arrayorder = ArrayTypeorder()
        for i in range(0, countorder):
            arrayorder[i] = 1 if order[i] else 0

        GetDllLibPdf().Collections_PdfCollection_Sort.argtypes = [c_void_p, ArrayTypefieldNames, c_int, ArrayTypeorder, c_int]
        CallCFunction(GetDllLibPdf().Collections_PdfCollection_Sort,self.Ptr, arrayfieldNames, countfieldNames, arrayorder, countorder)

    def Clear(self):
        """
        Clear all files and folders.
        """
        GetDllLibPdf().Collections_PdfCollection_Clear.argtypes = [c_void_p]
        CallCFunction(GetDllLibPdf().Collections_PdfCollection_Clear,self.Ptr)