from enum import Enum
from plum import dispatch
from typing import TypeVar, Union, Generic, List, Tuple
from spire.pdf.common import *
from spire.pdf import *
from ctypes import *
import abc

class PdfDestination(SpireObject):
    """
    Represents an anchor in the document where bookmarks or annotations can direct when clicked.
    """

    @dispatch
    def __init__(self, page: PdfPageBase):
        """
        Initializes a new instance of the PdfDestination class with the specified page.

        Args:
            page (PdfPageBase): The page where the destination is situated.
        """
        ptrPage: c_void_p = page.Ptr
        GetDllLibPdf().PdfDestination_CreatePdfDestinationP.argtypes = [c_void_p]
        GetDllLibPdf().PdfDestination_CreatePdfDestinationP.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfDestination_CreatePdfDestinationP,ptrPage)
        super(PdfDestination, self).__init__(intPtr)

    @dispatch
    def __init__(self, page: PdfPageBase, location: PointF):
        """
        Initializes a new instance of the PdfDestination class with the specified page and location.

        Args:
            page (PdfPageBase): The page where the destination is situated.
            location (PointF): The location of the destination.
        """
        ptrPage: c_void_p = page.Ptr
        ptrLocation: c_void_p = location.Ptr
        GetDllLibPdf().PdfDestination_CreatePdfDestinationPL.argtypes = [c_void_p, c_void_p]
        GetDllLibPdf().PdfDestination_CreatePdfDestinationPL.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfDestination_CreatePdfDestinationPL,ptrPage, ptrLocation)
        super(PdfDestination, self).__init__(intPtr)

    @dispatch
    def __init__(self, page: PdfPageBase, rectangle: RectangleF):
        """
        Initializes a new instance of the PdfDestination class with the specified page and rectangle.

        Args:
            page (PdfPageBase): The page where the destination is situated.
            rectangle (RectangleF): The rectangle of the destination.
        """
        ptrPage: c_void_p = page.Ptr
        ptrRec: c_void_p = rectangle.Ptr
        GetDllLibPdf().PdfDestination_CreatePdfDestinationPR.argtypes = [c_void_p, c_void_p]
        GetDllLibPdf().PdfDestination_CreatePdfDestinationPR.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfDestination_CreatePdfDestinationPR,ptrPage, ptrRec)
        super(PdfDestination, self).__init__(intPtr)

    @dispatch
    def __init__(self, pageNumber: int, location: PointF, zoom: float):
        """
        Initializes a new instance of the PdfDestination class with the specified page number, location, and zoom factor.

        Args:
            pageNumber (int): The zero-based page number.
            location (PointF): The location of the destination.
            zoom (float): The zoom factor.
        """
        ptrLocation: c_void_p = location.Ptr

        GetDllLibPdf().PdfDestination_CreatePdfDestinationPLZ.argtypes = [c_int, c_void_p, c_float]
        GetDllLibPdf().PdfDestination_CreatePdfDestinationPLZ.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfDestination_CreatePdfDestinationPLZ,pageNumber, ptrLocation, zoom)
        super(PdfDestination, self).__init__(intPtr)

    @property
    def PageNumber(self) -> int:
        """
        Gets the zero-based page number.

        Returns:
            int: The zero-based page number.
        """
        GetDllLibPdf().PdfDestination_get_PageNumber.argtypes = [c_void_p]
        GetDllLibPdf().PdfDestination_get_PageNumber.restype = c_int
        ret = CallCFunction(GetDllLibPdf().PdfDestination_get_PageNumber,self.Ptr)
        return ret

    @property
    def Zoom(self) -> float:
        """
        Gets or sets the zoom factor.

        Returns:
            float: The zoom factor.
        """
        GetDllLibPdf().PdfDestination_get_Zoom.argtypes = [c_void_p]
        GetDllLibPdf().PdfDestination_get_Zoom.restype = c_float
        ret = CallCFunction(GetDllLibPdf().PdfDestination_get_Zoom,self.Ptr)
        return ret

    @Zoom.setter
    def Zoom(self, value: float):
        """
        Sets the zoom factor.

        Args:
            value (float): The zoom factor.
        """
        GetDllLibPdf().PdfDestination_set_Zoom.argtypes = [c_void_p, c_float]
        CallCFunction(GetDllLibPdf().PdfDestination_set_Zoom,self.Ptr, value)

    @property
    def Page(self) -> 'PdfPageBase':
        """
        Gets or sets the page where the destination is situated.

        Returns:
            PdfPageBase: The page where the destination is situated.
        """
        GetDllLibPdf().PdfDestination_get_Page.argtypes = [c_void_p]
        GetDllLibPdf().PdfDestination_get_Page.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfDestination_get_Page,self.Ptr)
        ret = None if intPtr == None else PdfPageBase(intPtr)
        return ret

    @Page.setter
    def Page(self, value: 'PdfPageBase'):
        """
        Sets the page where the destination is situated.

        Args:
            value (PdfPageBase): The page where the destination is situated.
        """
        GetDllLibPdf().PdfDestination_set_Page.argtypes = [c_void_p, c_void_p]
        CallCFunction(GetDllLibPdf().PdfDestination_set_Page,self.Ptr, value.Ptr)

    @property
    def Mode(self) -> 'PdfDestinationMode':
        """
        Gets or sets the mode of the destination.

        Returns:
            PdfDestinationMode: The mode of the destination.
        """
        GetDllLibPdf().PdfDestination_get_Mode.argtypes = [c_void_p]
        GetDllLibPdf().PdfDestination_get_Mode.restype = c_int
        ret = CallCFunction(GetDllLibPdf().PdfDestination_get_Mode,self.Ptr)
        objwrapped = PdfDestinationMode(ret)
        return objwrapped

    @Mode.setter
    def Mode(self, value: 'PdfDestinationMode'):
        """
        Sets the mode of the destination.

        Args:
            value (PdfDestinationMode): The mode of the destination.
        """
        GetDllLibPdf().PdfDestination_set_Mode.argtypes = [c_void_p, c_int]
        CallCFunction(GetDllLibPdf().PdfDestination_set_Mode,self.Ptr, value.value)

    @property
    def Location(self) -> 'PointF':
        """
        Gets or sets the location of the destination.

        Returns:
            PointF: The location of the destination.
        """
        GetDllLibPdf().PdfDestination_get_Location.argtypes = [c_void_p]
        GetDllLibPdf().PdfDestination_get_Location.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfDestination_get_Location,self.Ptr)
        ret = None if intPtr == None else PointF(intPtr)
        return ret

    @Location.setter
    def Location(self, value: 'PointF'):
        """
        Sets the location of the destination.

        Args:
            value (PointF): The location of the destination.
        """
        GetDllLibPdf().PdfDestination_set_Location.argtypes = [c_void_p, c_void_p]
        CallCFunction(GetDllLibPdf().PdfDestination_set_Location,self.Ptr, value.Ptr)

    @property
    def Rectangle(self) -> 'RectangleF':
        """
        Gets or sets the rectangle of the destination.

        Returns:
            RectangleF: The rectangle of the destination.
        """
        GetDllLibPdf().PdfDestination_get_Rectangle.argtypes = [c_void_p]
        GetDllLibPdf().PdfDestination_get_Rectangle.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfDestination_get_Rectangle,self.Ptr)
        ret = None if intPtr == None else RectangleF(intPtr)
        return ret

    @Rectangle.setter
    def Rectangle(self, value: 'RectangleF'):
        """
        Sets the rectangle of the destination.

        Args:
            value (RectangleF): The rectangle of the destination.
        """
        GetDllLibPdf().PdfDestination_set_Rectangle.argtypes = [c_void_p, c_void_p]
        CallCFunction(GetDllLibPdf().PdfDestination_set_Rectangle,self.Ptr, value.Ptr)

    @property
    def IsValid(self) -> bool:
        """
        Gets a value indicating whether this instance is valid.

        Returns:
            bool: True if this instance is valid; otherwise, False.
        """
        GetDllLibPdf().PdfDestination_get_IsValid.argtypes = [c_void_p]
        GetDllLibPdf().PdfDestination_get_IsValid.restype = c_bool
        ret = CallCFunction(GetDllLibPdf().PdfDestination_get_IsValid,self.Ptr)
        return ret