from enum import Enum
from plum import dispatch
from typing import TypeVar, Union, Generic, List, Tuple
from spire.pdf.common import *
from spire.pdf import *
from ctypes import *
import abc

class PdfNewDocument(PdfDocumentBase):
    @dispatch
    def __init__(self):
        """
        Represents a logic to create Pdf document.
        """
        GetDllLibPdf().PdfNewDocument_CreatePdfNewDocument.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfNewDocument_CreatePdfNewDocument)
        super(PdfNewDocument, self).__init__(intPtr)

    @property
    def Bookmarks(self) -> 'PdfBookmarkCollection':
        """
        Gets the root of the bookmark tree in the document.
        :return: A PdfBookmarkCollection object specifying the document's bookmarks.
        :remarks: Creates a bookmark root instance if it's called for the first time.
        """
        GetDllLibPdf().PdfNewDocument_get_Bookmarks.argtypes = [c_void_p]
        GetDllLibPdf().PdfNewDocument_get_Bookmarks.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfNewDocument_get_Bookmarks,self.Ptr)
        ret = None if intPtr == None else PdfBookmarkCollection(intPtr)
        return ret

    @property
    def Attachments(self) -> 'PdfAttachmentCollection':
        """
        Gets the attachments of the document.
        :return: The PdfAttachmentCollection object contains a list of files which are attached in the PDF document.
        """
        GetDllLibPdf().PdfNewDocument_get_Attachments.argtypes = [c_void_p]
        GetDllLibPdf().PdfNewDocument_get_Attachments.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfNewDocument_get_Attachments,self.Ptr)
        ret = None if intPtr == None else PdfAttachmentCollection(intPtr)
        return ret

    @property
    def Form(self) -> 'PdfForm':
        """
        Gets the interactive form of the document.
        :return: The PdfForm object contains the list of form elements of the document.
        """
        GetDllLibPdf().PdfNewDocument_get_Form.argtypes = [c_void_p]
        GetDllLibPdf().PdfNewDocument_get_Form.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfNewDocument_get_Form,self.Ptr)
        ret = None if intPtr == None else PdfForm(intPtr)
        return ret

    @property
    def ColorSpace(self) -> 'PdfColorSpace':
        """
        Gets or sets the color space of the document.
        :return: The PdfColorSpace of the document.
        :remarks: This property has an impact on the newly created pages only.
        If a page was created, it remains its color space obliviously to this property changes.
        """
        GetDllLibPdf().PdfNewDocument_get_ColorSpace.argtypes = [c_void_p]
        GetDllLibPdf().PdfNewDocument_get_ColorSpace.restype = c_int
        ret = CallCFunction(GetDllLibPdf().PdfNewDocument_get_ColorSpace,self.Ptr)
        objwrapped = PdfColorSpace(ret)
        return objwrapped

    @ColorSpace.setter
    def ColorSpace(self, value: 'PdfColorSpace'):
        GetDllLibPdf().PdfNewDocument_set_ColorSpace.argtypes = [c_void_p, c_int]
        CallCFunction(GetDllLibPdf().PdfNewDocument_set_ColorSpace,self.Ptr, value.value)

    @property
    def Conformance(self) -> 'PdfConformanceLevel':
        """
        Gets or Sets the Pdf Conformance level.
        Supported: PDF/A-1b - Level B compliance in Part 1
        """
        GetDllLibPdf().PdfNewDocument_get_Conformance.argtypes = [c_void_p]
        GetDllLibPdf().PdfNewDocument_get_Conformance.restype = c_int
        ret = CallCFunction(GetDllLibPdf().PdfNewDocument_get_Conformance,self.Ptr)
        objwrapped = PdfConformanceLevel(ret)
        return objwrapped

    def Save(self, stream: 'Stream'):
        """
        Saves the document to the specified stream.
        :param stream: The stream object where PDF document will be saved.
        """
        intPtrstream: c_void_p = stream.Ptr

        GetDllLibPdf().PdfNewDocument_Save.argtypes = [c_void_p, c_void_p]
        CallCFunction(GetDllLibPdf().PdfNewDocument_Save,self.Ptr, intPtrstream)

    def Close(self, completely: bool):
        """
        Closes the document.
        :param completely: If set to True, the document should be disposed completely.
        :remarks: The document is disposed after calling the Close method.
        So, the document cannot be saved if Close method was invoked.
        """
        GetDllLibPdf().PdfNewDocument_Close.argtypes = [c_void_p, c_bool]
        CallCFunction(GetDllLibPdf().PdfNewDocument_Close,self.Ptr, completely)

    def Clone(self) -> 'SpireObject':
        """
        Creates a new object that is a copy of the current instance.
        :return: A new object that is a copy of this instance.
        :remarks: The resulting clone must be of the same type as or a compatible type to the original instance.
        """
        GetDllLibPdf().PdfNewDocument_Clone.argtypes = [c_void_p]
        GetDllLibPdf().PdfNewDocument_Clone.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfNewDocument_Clone,self.Ptr)
        ret = None if intPtr == None else SpireObject(intPtr)
        return ret