from enum import Enum
from plum import dispatch
from typing import TypeVar, Union, Generic, List, Tuple
from spire.pdf.common import *
from spire.pdf import *
from ctypes import *
import abc

class PdfStructureElement(SpireObject):
    """
    Represents the pdf structure element.
    """

    @property
    def Title(self) -> str:
        """
        The title of the structure element.
        """
        GetDllLibPdf().PdfStructureElement_get_Title.argtypes = [c_void_p]
        GetDllLibPdf().PdfStructureElement_get_Title.restype = c_void_p
        ret = PtrToStr(CallCFunction(GetDllLibPdf().PdfStructureElement_get_Title,self.Ptr))
        return ret

    @Title.setter
    def Title(self, value: str):
        GetDllLibPdf().PdfStructureElement_set_Title.argtypes = [c_void_p, c_wchar_p]
        CallCFunction(GetDllLibPdf().PdfStructureElement_set_Title,self.Ptr, value)

    @property
    def Alt(self) -> str:
        """
        An alternate description of the structure element and its children in human-readable form, which is useful 
        when extracting the document’s contents in support of accessibility to users with disabilities or for other purposes.
        """
        GetDllLibPdf().PdfStructureElement_get_Alt.argtypes = [c_void_p]
        GetDllLibPdf().PdfStructureElement_get_Alt.restype = c_void_p
        ret = PtrToStr(CallCFunction(GetDllLibPdf().PdfStructureElement_get_Alt,self.Ptr))
        return ret

    @Alt.setter
    def Alt(self, value: str):
        GetDllLibPdf().PdfStructureElement_set_Alt.argtypes = [c_void_p, c_wchar_p]
        CallCFunction(GetDllLibPdf().PdfStructureElement_set_Alt,self.Ptr, value)

    @property
    def ActualText(self) -> str:
        """
        Text that is an exact replacement for the structure element and its children. This replacement text (which should apply 
        to as small a piece of content as possible) is useful when extracting the document’s contents in support of accessibility 
        to users with disabilities or for other purposes.
        """
        GetDllLibPdf().PdfStructureElement_get_ActualText.argtypes = [c_void_p]
        GetDllLibPdf().PdfStructureElement_get_ActualText.restype = c_void_p
        ret = PtrToStr(CallCFunction(GetDllLibPdf().PdfStructureElement_get_ActualText,self.Ptr))
        return ret

    @ActualText.setter
    def ActualText(self, value: str):
        GetDllLibPdf().PdfStructureElement_set_ActualText.argtypes = [c_void_p, c_wchar_p]
        CallCFunction(GetDllLibPdf().PdfStructureElement_set_ActualText,self.Ptr, value)

    def AppendChildElement(self, structureType: str) -> 'PdfStructureElement':
        """
        Append structure type element.
        :param structureType: The structure type.
        :return: The pdf structure type element.
        """
        GetDllLibPdf().PdfStructureElement_AppendChildElement.argtypes = [c_void_p, c_wchar_p]
        GetDllLibPdf().PdfStructureElement_AppendChildElement.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfStructureElement_AppendChildElement,self.Ptr, structureType)
        ret = None if intPtr == None else PdfStructureElement(intPtr)
        return ret

    @dispatch
    def GetAttributes(self, owner: PdfAttributeOwner) -> PdfStructureAttributes:
        """
        Get the owner's attributes.
        :param owner: The owner.
        :return: The owner's attributes.
        """
        intPtrowner: c_void_p = owner.Ptr

        GetDllLibPdf().PdfStructureElement_GetAttributesO.argtypes = [c_void_p, c_void_p]
        GetDllLibPdf().PdfStructureElement_GetAttributesO.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfStructureElement_GetAttributesO,self.Ptr, intPtrowner)
        ret = None if intPtr == None else PdfStructureAttributes(intPtr)
        return ret

    def AddAttributes(self, owner: 'PdfAttributeOwner') -> 'PdfStructureAttributes':
        """
        Add the owner's attributes.
        :param owner: The owner.
        :return: The owner's attributes.
        """
        intPtrowner: c_void_p = owner.Ptr

        GetDllLibPdf().PdfStructureElement_AddAttributes.argtypes = [c_void_p, c_void_p]
        GetDllLibPdf().PdfStructureElement_AddAttributes.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfStructureElement_AddAttributes,self.Ptr, intPtrowner)
        ret = None if intPtr == None else PdfStructureAttributes(intPtr)
        return ret

    @dispatch
    def BeginMarkedContent(self, page: PdfPageBase):
        """
        Begin a marked-content sequence of objects within the content stream.
        :param page: The pdf page.
        """
        intPtrpage: c_void_p = page.Ptr

        GetDllLibPdf().PdfStructureElement_BeginMarkedContent.argtypes = [c_void_p, c_void_p]
        CallCFunction(GetDllLibPdf().PdfStructureElement_BeginMarkedContent,self.Ptr, intPtrpage)

    @dispatch
    def BeginMarkedContent(self, page: PdfPageBase, artifactPropertyList: ArtifactPropertyList):
        """
        Begin a marked-content sequence of objects within the content stream.
        :param page: The pdf page.
        :param artifactPropertyList: The artifact property list.
        """
        intPtrpage: c_void_p = page.Ptr
        intPtrartifactPropertyList: c_void_p = artifactPropertyList.Ptr

        GetDllLibPdf().PdfStructureElement_BeginMarkedContentPA.argtypes = [c_void_p, c_void_p, c_void_p]
        CallCFunction(GetDllLibPdf().PdfStructureElement_BeginMarkedContentPA,self.Ptr, intPtrpage, intPtrartifactPropertyList)

    def EndMarkedContent(self, page: 'PdfPageBase'):
        """
        End a marked-content sequence of objects within the content stream.
        :param page: The pdf page.
        """
        intPtrpage: c_void_p = page.Ptr

        GetDllLibPdf().PdfStructureElement_EndMarkedContent.argtypes = [c_void_p, c_void_p]
        CallCFunction(GetDllLibPdf().PdfStructureElement_EndMarkedContent,self.Ptr, intPtrpage)