from enum import Enum
from plum import dispatch
from typing import TypeVar, Union, Generic, List, Tuple
from spire.pdf.common import *
from spire.pdf import *
from ctypes import *
import abc

class PdfTextBoxField(PdfAppearanceField, IPdfTextBoxField):
    """
    Represents text box field in the PDF form.
    """
    @dispatch
    def __init__(self, page: PdfPageBase, name: str):
        """
        Initializes a new instance of the PdfTextBoxField class.

        Args:
            page (PdfPageBase): The page to which the field belongs.
            name (str): The name of the field.
        """
        ptrPage: c_void_p = page.Ptr
        GetDllLibPdf().PdfTextBoxField_CreatePdfTextBoxFieldPN.argtypes = [c_void_p, c_wchar_p]
        GetDllLibPdf().PdfTextBoxField_CreatePdfTextBoxFieldPN.restype = c_void_p
        intPtr = CallCFunction(GetDllLibPdf().PdfTextBoxField_CreatePdfTextBoxFieldPN,ptrPage, name)
        super(PdfTextBoxField, self).__init__(intPtr)

    @property
    def Text(self) -> str:
        """
        Gets or sets the text.

        Returns:
            str: The text of the text box field.
        """
        GetDllLibPdf().PdfTextBoxField_get_Text.argtypes = [c_void_p]
        GetDllLibPdf().PdfTextBoxField_get_Text.restype = c_void_p
        ret = PtrToStr(CallCFunction(GetDllLibPdf().PdfTextBoxField_get_Text,self.Ptr))
        return ret

    @Text.setter
    def Text(self, value: str):
        """
        Sets the text of the text box field.

        Args:
            value (str): The text to set.
        """
        GetDllLibPdf().PdfTextBoxField_set_Text.argtypes = [c_void_p, c_wchar_p]
        CallCFunction(GetDllLibPdf().PdfTextBoxField_set_Text,self.Ptr, value)

    @property
    def DefaultValue(self) -> str:
        """
        Gets or sets the default value.

        Returns:
            str: The default value of the text box field.
        """
        GetDllLibPdf().PdfTextBoxField_get_DefaultValue.argtypes = [c_void_p]
        GetDllLibPdf().PdfTextBoxField_get_DefaultValue.restype = c_void_p
        ret = PtrToStr(CallCFunction(GetDllLibPdf().PdfTextBoxField_get_DefaultValue,self.Ptr))
        return ret

    @DefaultValue.setter
    def DefaultValue(self, value: str):
        """
        Sets the default value of the text box field.

        Args:
            value (str): The default value to set.
        """
        GetDllLibPdf().PdfTextBoxField_set_DefaultValue.argtypes = [c_void_p, c_wchar_p]
        CallCFunction(GetDllLibPdf().PdfTextBoxField_set_DefaultValue,self.Ptr, value)

    @property
    def SpellCheck(self) -> bool:
        """
        Gets or sets a value indicating whether to check spelling.

        Returns:
            bool: True if check spelling; otherwise, False.
        """
        GetDllLibPdf().PdfTextBoxField_get_SpellCheck.argtypes = [c_void_p]
        GetDllLibPdf().PdfTextBoxField_get_SpellCheck.restype = c_bool
        ret = CallCFunction(GetDllLibPdf().PdfTextBoxField_get_SpellCheck,self.Ptr)
        return ret

    @SpellCheck.setter
    def SpellCheck(self, value: bool):
        """
        Sets a value indicating whether to check spelling.

        Args:
            value (bool): True to check spelling; otherwise, False.
        """
        GetDllLibPdf().PdfTextBoxField_set_SpellCheck.argtypes = [c_void_p, c_bool]
        CallCFunction(GetDllLibPdf().PdfTextBoxField_set_SpellCheck,self.Ptr, value)

    @property
    def InsertSpaces(self) -> bool:
        """
        Meaningful only if the MaxLength property is set and the Multiline, Password properties are false.
        If set, the field is automatically divided into as many equally spaced positions, or combs,
        as the value of MaxLength, and the text is laid out into those combs.

        Returns:
            bool: True if need to insert spaces; otherwise, False.
        """
        GetDllLibPdf().PdfTextBoxField_get_InsertSpaces.argtypes = [c_void_p]
        GetDllLibPdf().PdfTextBoxField_get_InsertSpaces.restype = c_bool
        ret = CallCFunction(GetDllLibPdf().PdfTextBoxField_get_InsertSpaces,self.Ptr)
        return ret

    @InsertSpaces.setter
    def InsertSpaces(self, value: bool):
        """
        Sets a value indicating whether to insert spaces.

        Args:
            value (bool): True to insert spaces; otherwise, False.
        """
        GetDllLibPdf().PdfTextBoxField_set_InsertSpaces.argtypes = [c_void_p, c_bool]
        CallCFunction(GetDllLibPdf().PdfTextBoxField_set_InsertSpaces,self.Ptr, value)

    @property
    def Multiline(self) -> bool:
        """
        Gets or sets a value indicating whether this is multiline.

        Returns:
            bool: True if multiline; otherwise, False.
        """
        GetDllLibPdf().PdfTextBoxField_get_Multiline.argtypes = [c_void_p]
        GetDllLibPdf().PdfTextBoxField_get_Multiline.restype = c_bool
        ret = CallCFunction(GetDllLibPdf().PdfTextBoxField_get_Multiline,self.Ptr)
        return ret

    @Multiline.setter
    def Multiline(self, value: bool):
        """
        Sets a value indicating whether this is multiline.

        Args:
            value (bool): True if multiline; otherwise, False.
        """
        GetDllLibPdf().PdfTextBoxField_set_Multiline.argtypes = [c_void_p, c_bool]
        CallCFunction(GetDllLibPdf().PdfTextBoxField_set_Multiline,self.Ptr, value)

    @property
    def Password(self) -> bool:
        """
        Gets or sets a value indicating whether this is a password field.

        Returns:
            bool: True if password field; otherwise, False.
        """
        GetDllLibPdf().PdfTextBoxField_get_Password.argtypes = [c_void_p]
        GetDllLibPdf().PdfTextBoxField_get_Password.restype = c_bool
        ret = CallCFunction(GetDllLibPdf().PdfTextBoxField_get_Password,self.Ptr)
        return ret

    @Password.setter
    def Password(self, value: bool):
        """
        Sets a value indicating whether this is a password field.

        Args:
            value (bool): True if password field; otherwise, False.
        """
        GetDllLibPdf().PdfTextBoxField_set_Password.argtypes = [c_void_p, c_bool]
        CallCFunction(GetDllLibPdf().PdfTextBoxField_set_Password,self.Ptr, value)

    @property
    def Scrollable(self) -> bool:
        """
        Gets or sets a value indicating whether this is scrollable.

        Returns:
            bool: True if scrollable; otherwise, False.
        """
        GetDllLibPdf().PdfTextBoxField_get_Scrollable.argtypes = [c_void_p]
        GetDllLibPdf().PdfTextBoxField_get_Scrollable.restype = c_bool
        ret = CallCFunction(GetDllLibPdf().PdfTextBoxField_get_Scrollable,self.Ptr)
        return ret

    @Scrollable.setter
    def Scrollable(self, value: bool):
        """
        Sets a value indicating whether this is scrollable.

        Args:
            value (bool): True if scrollable; otherwise, False.
        """
        GetDllLibPdf().PdfTextBoxField_set_Scrollable.argtypes = [c_void_p, c_bool]
        CallCFunction(GetDllLibPdf().PdfTextBoxField_set_Scrollable,self.Ptr, value)

    @property
    def MaxLength(self) -> int:
        """
        Gets or sets the maximum number of characters that can be entered in the text box.

        Returns:
            int: An integer value specifying the maximum number of characters that can be entered in the text box.
        """
        GetDllLibPdf().PdfTextBoxField_get_MaxLength.argtypes = [c_void_p]
        GetDllLibPdf().PdfTextBoxField_get_MaxLength.restype = c_int
        ret = CallCFunction(GetDllLibPdf().PdfTextBoxField_get_MaxLength,self.Ptr)
        return ret

    @MaxLength.setter
    def MaxLength(self, value: int):
        """
        Sets the maximum number of characters that can be entered in the text box.

        Args:
            value (int): An integer value specifying the maximum number of characters to set.
        """
        GetDllLibPdf().PdfTextBoxField_set_MaxLength.argtypes = [c_void_p, c_int]
        CallCFunction(GetDllLibPdf().PdfTextBoxField_set_MaxLength,self.Ptr, value)
